package com.txtnation.inappdemo;

import java.util.List;
import java.util.UUID;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.View;

import com.txtnation.android.BillingManager;
import com.txtnation.android.Environment;
import com.txtnation.android.PaymentListener;
import com.txtnation.android.products.Product;

/**
 * This class extends Activity and should be locked to being "portrait mode" in
 * the manifest file so that the layout code doesn't get confused especially if
 * the list of products is lengthy.
 * 
 * If you want to see screenshots of this application in action to get a feel
 * for how it should work then please read this wiki page:
 * 
 * http://wiki.txtnation.com/wiki/In-App_Dialog_Sequence
 * 
 * @author txtNation
 */
public class MainActivity extends Activity implements PaymentListener {
	/**
	 * This will hold the instance of the billing manager; this demo code is
	 * running in "one-step" mode which means that internal code in the library
	 * will automatically display the returned list of products and handle the
	 * selection and confirmation.
	 */
	private BillingManager billingManager;

	/**
	 * All UI updates from threads MUST be done using one of the prescribed
	 * Android techniques; the BillingManager requires a Handler be given to it so
	 * that it can perform its duties, for example, displaying the busy dialogs
	 * and user input screens.
	 */
	private final Handler mHandler = new Handler();

	/**
	 * This flag is used to store the fact that we may already be in the process
	 * of running the in-app. billing dialog. It is set when the BillingManager
	 * successfully starts a payment cycle. You can use it to ensure that your
	 * application doesn't try to start a second instance should one be in
	 * progress and not yet completed.
	 */
	private boolean started = false;

	/**
	 * This flag is for internal testing / development. When set to false it means
	 * that your application is not ready to use the "production" mode of the
	 * library but instead will revert to "LOCAL" mode which means that you can
	 * use two emulator instances as documented on the txtNation wiki here:
	 * 
	 * http://wiki.txtnation.com/wiki/Using_Two_Android_Emulators
	 */
	private final boolean isProduction = false;

	/**
	 * As well as the usual unpacking and inflating the view, this code also
	 * checks the "isProduction" state flag to see if the normal operation or
	 * "local mode" operation is required. If the local mode is required then it
	 * primes the test number feature ""
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_main);

		billingManager = new BillingManager(MainActivity.this, mHandler);

		if (isProduction) {
			billingManager.setEnvironment(Environment.PRODUCTION);
		} else {
			// ! Set the billing manager to operate in a development / debugging mode
			// ! such that a second emulator instance can be used to (a) receive a
			// ! message from this application and then (b) send it back. For full
			// ! details on this please refer to the wiki page at:
			// !
			// ! http://wiki.txtnation.com/wiki/Using_Two_Android_Emulators
			// !
			billingManager.setEnvironment(Environment.LOCAL);

			// ! 5556 *should* be the second emulator port number. If you didn't
			// ! already know, you can use "telnet" to talk to the emulators, see
			// ! here:
			// !
			// ! http://developer.android.com/tools/devices/emulator.html#console
			// !
			billingManager.setTestNumber("5556");
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	/**
	 * This is the button handler for the demo application, as configured in the
	 * "activity_main.xml" file. For our purposes this is how we start the demo
	 * but in reality you will probably launch a new Intent containing your
	 * PaymentActivity implementation and do what you need to do.
	 * 
	 * @param view
	 * @throws Exception
	 */
	public void launchBillingProcess(View view) throws Exception {

		String strTxnId = UUID.randomUUID().toString();

		// ! "beginPayment" is the call to start the whole cycle; it requires a few
		// ! parameters, each of which is detailed below.
		// !
		started = billingManager.beginPayment(
		// ! 12345678 will be replaced by your accountId once you have completed
		// ! the sign-up process. The value 100001L is an internal indicator
		// ! that we are in debug / develop mode, not production mode.
		// !
				isProduction ? 12345678 : 100001L,
				// !
				// ! This MUST be unique on a live production system or you will get
				// ! failures. For testing however you could re-use the same string but
				// ! generating a UUID is just as simple.
				// !
				strTxnId,
				// !
				// ! This is the "clientAppVersion" string, here we just pull it from
				// ! the manifest. All applications intended for release on the Play
				// ! store MUST have a version string as well as the application name in
				// ! the manifest.
				// !
				getPackageManager().getPackageInfo(getPackageName(), 0).versionName,
				// !
				// ! This is the "isOneStepBilling" flag. In our demo it is TRUE to let
				// ! the TAL.jar library handle the product display and selection.
				// !
				true);
	}

	// =========================================================================
	// INTERFACE IMPLEMENTATION: PaymentListener
	// =========================================================================

	/**
	 * When the device has received the SMS message with the verification code,
	 * this callback will be invoked. At this point you can enable whatever
	 * feature, service etc. that the user has just purchased.
	 */
	@Override
	public void onSuccess() {
		// ! This is made up to satisfy the API call... in reality you would
		// ! remember the value used in the call to the "beginPayment()" call that
		// ! you made earlier.
		// !
		String clientTransactionId = "EXAMPLE3393939";

		// !
		// ! Setting this boolean is merely indicative of the location of payment
		// ! being received. In reality you would now use "sqlite" or some other
		// ! means to record the fact that the user has purchased something for when
		// ! the application is run again.
		// !
		boolean userAccountToppedUp = true;

		if (userAccountToppedUp) {
			billingManager.sendAcknowledgementSuccess(clientTransactionId);
		} else {
			billingManager.sendAcknowledgementFailure(clientTransactionId,
					"Reason why your system could not top up users account");
		}
	}

	@Override
	public void onCancelled() {
	}

	@Override
	public void onFailure(String arg0) {
	}

	@Override
	public void onPaymentStateChanged(String arg0) {
	}

	@Override
	public void onProductSelected(Product p) {
	}

	@Override
	public void onProductsReceived(List<Product> arg0) {
	}

}
